/*
Nixie driver Arduino Project
Interfaces to one or more MAX6922 32 bit HV drivers
using simple serial bit banging of 3 digital outputs:
data, clock and latch.
Each MAX6922 can drive 3 Nixies, and can be daisy-chained to drive
an unlimited number of them. 
Each driver has 32 outputs which leaves two leftover bits per driver
which can be used for a partial digit or for colon or decimal points.

A wiring reRoute array is included to allow for easier wiring between
the HV driver and Nixies, so that the cathodes don't necessarily
have to connect in numerical order. This is especially handy for IN-16's
which have a non-sequential pinout.
*/

//Global data

// Nixie specific variables
const int nNixies = 3; //Number of nixies to be driven
int reRoute[10] = {5,4,8,2,1,0,9,3,7,6}; // IN-16 reroute
int myDigits[nNixies]; // digit array - dimension according to the number of nixies being driven
int msd = 0; // The "one" digit for a half digit display
// Nixie driver data lines:
const int nixData = 10;
const int nixLatch = 11;
const int nixClk = 12;
//
// Project specific:
//
// blinky LED
const int led = 13;
int ledState = 0;
// analog
const int vInput = A0;

void nixIOinit() {
  digitalWrite(nixLatch,LOW);
  digitalWrite(nixClk,LOW);
  digitalWrite(nixData,LOW);
}

void nixBitOut(int dataBit) {
  //set the data bit
  if (dataBit==1) {
    digitalWrite(nixData,HIGH);
  }
  else {
    digitalWrite(nixData,LOW);
  }
  //Now toggle the clock line
  digitalWrite(nixClk,HIGH);
  digitalWrite(nixClk,LOW);
}

void nixPush() {
  //Latch the shift reg data to the parallel output drivers
  digitalWrite(nixLatch, HIGH);
  digitalWrite(nixLatch, LOW);
}

void nixOut(byte miscBits, int nDigits, int digits[] ) {
  /*
    Send out 8 miscellaneous bits (incomplete digit), followed by
    nDigits digits. If miscBits and digits[] contain more total bits
    than are available in the HV driver(s) then they disappear out the 
    end of the last shift register, and only the least significant
    bits remain in the driver.

    Note - The original display board for this project had the colon
    neon lamp connected to output #20 of the HV driver, between Nixie
    digits 2 and 3. If the colon is omitted and the Nixies are connected
    to the driver outputs with no gaps, then the 'if (i==1) nixBitOut(1);'
    statement in the following function should be deleted or commented out.
  */
  //Start with the miscellaneous bits
  byte bShift = miscBits;
  for (int i=0;i<8;i++) {
    nixBitOut(bShift % 2);
    bShift /= 2;
  }
  //Now do the complete digits
  for (int i=nDigits-1;i>=0;i--) {
    if (i==1) nixBitOut(1); //skip past colon position. See note above.
    for (int j=0;j<10;j++) {
      nixBitOut((reRoute[digits[i]]!=j)?1:0);
    }
  }
  nixPush();
}

void setup() {
  pinMode(led, OUTPUT);  
  pinMode(nixData, OUTPUT);  
  pinMode(nixClk, OUTPUT);  
  pinMode(nixLatch, OUTPUT); 
  nixIOinit();
  for (int i=0;i<nNixies;i++){
    myDigits[i] = 0;
  }
  nixOut(255,nNixies,myDigits);
  delay(2000); //This delay for testing display init only; can be removed
}


void loop() {
  //Blink the LED so we know something's happening 
  // (just in case nothing else works)
  delay(50);
  ledState = 1 - ledState;
  digitalWrite(led,ledState);
  
  int vCount = map(analogRead(vInput),0,1024,0,2000);   
  msd = vCount/1000; // most significant half digit
  for (int i=0;i<3;i++) {
    myDigits[i] = vCount % 10;
    vCount /= 10;
  }
  nixOut(128*(1-msd),3,myDigits);
}


